@@ -1,5 +1,5 @@
 /*
- * Copyright (C)2005-2012 Haxe Foundation
+ * Copyright (C)2005-2017 Haxe Foundation
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -172,7 +172,7 @@ static value result_next( value o ) {
 				value v;
 				switch( r->fields_convs[i] ) {
 				case CONV_INT:
-					v = alloc_int(atoi(row[i]));
+					v = alloc_best_int(atoi(row[i]));
 					break;
 				case CONV_STRING:
 					v = alloc_string(row[i]);
@@ -450,6 +450,7 @@ static value escape( value o, value s ) {
 		bfailure(b);
 	}
 	val_set_length(sout,len);
+	val_string(sout)[len] = 0;
 	return sout;
 }
 
@@ -480,7 +481,7 @@ static void free_connection( value o ) {
 	connect : { host => string, port => int, user => string, pass => string, socket => string? } -> 'connection
 	<doc>Connect to a database using the connection informations</doc>
 **/
-static value connect( value params  ) {
+static value connect_mysql( value params  ) {
 	value host, port, user, pass, socket;
 	val_check(params,object);
 	host = val_field(params,val_id("host"));
@@ -516,7 +517,7 @@ static value connect( value params  ) {
 // ---------------------------------------------------------------
 // Registers
 
-DEFINE_PRIM(connect,1);
+DEFINE_PRIM_WITH_NAME(connect_mysql,connect,1);
 DEFINE_PRIM(close,1);
 DEFINE_PRIM(request,2);
 DEFINE_PRIM(select_db,2);
