@@ -1,5 +1,5 @@
 /*
- * Copyright (C)2005-2012 Haxe Foundation
+ * Copyright (C)2005-2017 Haxe Foundation
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -42,6 +42,7 @@ static INLINE int icmp( int a, int b ) {
 }
 
 static INLINE int fcmp( tfloat a, tfloat b ) {
+	if(a != a || b != b) return invalid_comparison;
 	return (a == b)?0:((a < b)?-1:1);
 }
 
@@ -145,7 +146,7 @@ static void buffer_append_new( buffer b, const char *s, int len ) {
 	it->size = size;
 	it->len = len;
 	it->next = b->data;
-	b->data = it;	
+	b->data = it;
 }
 
 EXTERN void buffer_append_sub( buffer b, const char *s, int_val _len ) {
@@ -182,7 +183,7 @@ EXTERN void buffer_append_char( buffer b, char c ) {
 	b->totlen++;
 	it = b->data;
 	if( it && it->len != it->size ) {
-		it->str[it->len++] = c;		
+		it->str[it->len++] = c;
 		return;
 	}
 	buffer_append_new(b,&c,1);
@@ -202,6 +203,10 @@ EXTERN value buffer_to_string( buffer b ) {
 	return v;
 }
 
+EXTERN int buffer_length( buffer b ) {
+	return b->totlen;
+}
+
 typedef struct vlist {
 	value v;
 	struct vlist *next;
@@ -411,22 +416,15 @@ EXTERN field val_id( const char *name ) {
 		}
 		// in case we found it, it means that it's been inserted by another thread
 		if( fdata == val_null ) {
-			objcell *c2 = (objcell*)alloc(sizeof(objcell)*(t->count+1));
+			const size_t objcell_size = sizeof(objcell);
+			objcell *c2 = (objcell*)alloc(objcell_size * (t->count + 1));
 
 			// copy the whole table
 			mid = (min + max) >> 1;
-			min = 0;
-			while( min < mid ) {
-				c2[min] = c[min];
-				min++;
-			}
-			c2[min].id = f;
-			c2[min].v = copy_string(oname,name - oname);
-			max = t->count;
-			while( min < max ) {
-				c2[min+1] = c[min];
-				min++;
-			}
+			memcpy(c2, c, mid * objcell_size);
+			c2[mid].id = f;
+			c2[mid].v = copy_string(oname,name - oname);
+			memcpy(&c2[mid + 1], &c[mid], (t->count - mid) * objcell_size);
 
 			// update
 			t->cells = c2;
@@ -450,8 +448,6 @@ EXTERN value val_field_name( field id ) {
 
 EXTERN value val_field( value _o, field id ) {
 	value *f;
-	// WARNING : we can't change the value on the stack
-	// since it will be reused by the JIT (when compiled with GCC)
 	vobject *o = (vobject*)_o;
 	do {
 		f = otable_find(&o->table,id);
